@@ -25,6 +25,7 @@
 #include <ctype.h>
 #include <pds/pdsfilesfunc.h>
 #include <pds/pdscstring.h>
+#include <pds/pdsstring.h>
 
 #include <fcntl.h>           /* Definition of AT_* constants */
 #include <unistd.h>
@@ -34,13 +35,21 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include <time.h>
+
 /** \fn PdsCellString *pds_get_arquives_new(const char *dirpath,ArqType Type,const char* pattern_include,const char* pattern_exclude)
- *  \brief Retorna una estructura PdsCellString con un arreglo de células de nombres de arquivo.
+ *  \brief Retorna los archivos dentro de un directorio (siguiendo algunos criterios).
+ *
+ *  Retorna una estructura PdsCellString con un arreglo de células de nombres de arquivo.
  *  A partir de aqui pueden ser leidos los siguientes con la función pds_cell_string_read().
  *  \param[in] dirpath Camino del directorio en consulta.
- *  \param[in] Type El tipo de arquivo a leer, pueden ser los valores TYPE_DIR
- *  para directorios, TYPE_DIRTOP para directorios incluyendo la raiz o TYPE_FILE 
- *  para fiheros. Cualquier otro valor es equivalente a usar TYPE_FILE.
+ *  \param[in] Type El tipo de arquivo a leer, pueden ser usados los valores :
+ <ul>
+    <li><b>TYPE_DIR</b> para directorios, </li>
+    <li><b>TYPE_DIRTOP</b> para directorios incluyendo la raiz o </li>
+    <li><b>TYPE_FILE</b>  para fiheros.</li> 
+    <li>Cualquier otro valor es equivalente a usar TYPE_FILE.</li>
+ </ul>
  *  \param[in] pattern_include Patrón de archivos a incluir solamente.
  *  \param[in] pattern_exclude Patrón de archivos a excluir.
  *  \return Retorna una estructura PdsCellString con un arreglo de células con 
@@ -508,7 +517,92 @@ char *pds_get_dirname(const char * file_path)
 }
 
 
+/** \fn char *pds_get_filepath_with_extension(const char *file_path,const char *extension)
+ *  \brief Obtén un filepath con el mismo filename pero con la extención cambiada 
+ *  a extension.
+ *
+ *  Si la ruta de fichero es: /path_to_file/filename.c
+ *  La función retorna:       /path_to_file/filename.[extension]
+ *  \param[in] file_path Ruta del fichero a analizar.
+ *  \param[in] extension Extención a usar.
+ *  \return Retorna una nueva cadena de texto con el filepath nuevo, o 
+ *  NULL en caso de error de reserva de memoria.
+ *  \ingroup PdsFilesFuncGroup
+ */
+char *pds_get_filepath_with_extension(const char *file_path,const char *extension)
+{
+    char *cad=NULL;
+    char *dirname=NULL;
+    char *filename=NULL;
 
+    dirname=pds_get_dirname(file_path);
+
+    filename=pds_get_filename(file_path);
+    if(filename==NULL)
+    {
+        free(dirname);
+        return NULL;
+    }
+
+    if(dirname==NULL)
+        cad=pds_sprintf("%s.%s",filename,extension);
+    else
+        cad=pds_sprintf("%s%c%s.%s",dirname,pds_filesep(),filename,extension);
+
+    free(dirname);
+    free(filename);
+    return cad;
+}
+
+/** \fn char *pds_get_temporal_filepath(const char *filepath,const char *groupname)
+ *  \brief Obtén un nombre de archivo con la misma extención y directorio pero con nombre de
+ *  archivo diferente y aleatoriamente diseñado.
+ *
+ *  Si la ruta de fichero es: /path_to_file/filename.c
+ *  La función retorna:       /path_to_file/filename_[groupname][clock()][rand()].c
+ *  
+ *  \param[in] filepath Ruta del fichero a analizar. Esta ruta no precisa existir.
+ *  \param[in] groupname nombre que se agregará al nombre de archivo retornado.
+ *  \return Retorna una nueva cadena de texto con el filepath nuevo, o 
+ *  NULL en caso de error de reserva de memoria.
+ *  \ingroup PdsFilesFuncGroup
+ */
+char *pds_get_temporal_filepath(const char *filepath,const char *groupname)
+{
+    char *cad=NULL;
+    char *ext=NULL;
+    char *filename=NULL;
+    char *directory=NULL;
+
+    ext=pds_get_filename_extension(filepath);
+    if(ext==NULL)  return NULL;
+
+    filename=pds_get_filename(filepath);
+    if(filename==NULL)  
+    {
+        free(ext);
+        return NULL;
+    }
+
+    directory=pds_get_dirname(filepath);
+
+
+    if(directory==NULL)
+        if(groupname==NULL)
+            cad=pds_sprintf("%s_%ld%d.%s",filename,clock(),rand(),ext);
+        else
+            cad=pds_sprintf("%s_%s%ld%d.%s",filename,groupname,clock(),rand(),ext);
+    else
+        if(groupname==NULL)
+            cad=pds_sprintf("%s%c%s_%ld%d.%s",directory,pds_filesep(),filename,clock(),rand(),ext);
+        else
+            cad=pds_sprintf("%s%c%s_%s%ld%d.%s",directory,pds_filesep(),filename,groupname,clock(),rand(),ext);
+
+    free(ext);
+    free(filename);
+    free(directory);
+    return cad;
+}
 
 
 /** \fn PdsCellString *pds_get_all_filename_extensions(const char *dirpath,   const char* pattern_include,const char* pattern_exclude,const char* file_pattern_include,const char* file_pattern_exclude)
@@ -638,7 +732,7 @@ char *pds_fgets(FILE *fd)
  *
  *  \param[in] fd Descriptor de fichero de texto a analizar.
  *  \return Retorna una nueva cadena de texto con una copia de la linea leida 
- *  descartando los caracteres despues de '\n'.
+ *  descartando salto de linea '\\n'.
  *  \ingroup PdsFilesFuncGroup
  */
 char *pds_read_line(FILE *fd)
@@ -676,6 +770,94 @@ char *pds_read_line(FILE *fd)
     return data;
 }
 
+/** \fn char *pds_get_filedata_string(const char *pathname,const char* pattern)
+ *  \brief La funcion analiza linea por linea el archivo y retorna
+ *  el texto despues de el primer patrón pattern encontrado.
+ *
+ *  \param[in] pathname Fichero de texto a analizar.
+ *  \param[in] pattern Patrón a buscar.
+ *  \return Retorna una nueva cadena de texto con el dato encontrado.
+ *  \ingroup PdsFilesFuncGroup
+ */
+char *pds_get_filedata_string(const char *pathname,const char* pattern)
+{
+    FILE *fd=NULL;
+    char *LINE=NULL;
+    char *DATA=NULL;
+    int N,Nl,i;
+
+    if(pathname==NULL)  return NULL;
+    if(pattern ==NULL)  return NULL;
+
+    N=strlen(pattern);
+
+    fd=fopen(pathname,"r");
+    if(fd==NULL)
+    {
+        fprintf(stderr,"ERROR opening the file:%s\n",pathname);
+        return NULL;
+    }
+
+
+    while(feof(fd)==0)
+    {
+        free(LINE);
+        LINE=pds_read_line(fd);
+        if(LINE!=NULL)
+        {
+            Nl=strlen(LINE);
+            if(Nl>=N)
+            if(strncmp(pattern,LINE,N)==0)
+            {
+            
+
+                DATA=(char*)calloc(Nl-N+1,sizeof(char));
+                if(DATA==NULL)
+                {
+                    fprintf(stderr,"ERROR while alloc memory.\n");
+ 
+                }
+                else for(i=N;i<Nl;i++)   DATA[i-N]=LINE[i];
+
+                break;
+            }
+        }
+    }
+
+    fclose(fd);
+    free(LINE);
+
+    return DATA;
+}
+
+
+/** \fn PdsCellString *pds_get_filedata_cell_string(const char *pathname,const char* pattern,const char *separator)
+ *  \brief La funcion analiza linea por linea el archivo y retorna
+ *  el texto despues de el primer patrón pattern encontrado, Pero estos datos son
+ *  separados usando separator y ordenandos en una PdsCellString.
+ *
+ *  \param[in] pathname Fichero de texto a analizar.
+ *  \param[in] pattern Patrón a buscar.
+ *  \param[in] separator Separador de texto.
+ *  \return Retorna una nueva estructurs PdsCellString con los datos encontrado.
+ *  \ingroup PdsFilesFuncGroup
+ */
+PdsCellString *pds_get_filedata_cell_string(const char *pathname,const char* pattern,const char *separator)
+{
+    char *DATA=NULL;
+    PdsCellString *CELL=NULL;
+    
+    if(pathname==NULL)  return NULL;
+    if(pattern==NULL)   return NULL;
+
+    DATA=pds_get_filedata_string(pathname,pattern);
+    if(DATA==NULL)  return  NULL;
+
+    CELL=pds_strsep (DATA,separator);
+
+    free(DATA);
+    return CELL;
+}
 
 /** \fn int pds_is_file(const char *path)
  *  \brief TRUE si es un fichero (simbólico o real) o FALSE si no.
@@ -736,6 +918,39 @@ long pds_get_file_size(const char *path)
 }
 
 
+
+
+/** \fn long pds_get_number_of_chars(const char *path)
+ *  \brief Retorna el número de caracteres de un archivo.
+ *
+ *  \param[in] path Dirección de archivo a testar.
+ *  \return Retorna el número de caracteres de un archivo
+ *  o negativo en caso de error
+ *  \ingroup PdsFilesFuncGroup
+ */
+long pds_get_number_of_chars(const char *path)
+{
+    FILE *fd=NULL;
+    long count;
+    int c;
+
+    if(!pds_is_file(path))  return -1;
+
+
+    fd=fopen(path,"r");
+    if(fd==NULL)    return -1;
+
+    count=0;
+    while(c=fgetc(fd)) 
+    {
+        if(c == EOF) break;
+        count++;
+    }
+    fclose(fd);
+
+    return count;
+}
+
 /** \fn int pds_exist_file(const char *path)
  *  \brief Retorna TRUE si path es un fichero real o simbólico (no un directorio) 
  *  y es accesible.
@@ -776,3 +991,152 @@ int pds_archive_is_accessible(const char *path)
     }
 }
 
+
+
+/** \fn int pds_file_pattern_replacement(const char *filename_out,const char *filename_in,const char *pat,const char *rep)
+ *  \brief Esta función busca en un archivo de texto filename_in, un patron pat y los 
+ *  remplaza con el contenido de la cadena rep, el resultado es retornado en un 
+ *  archivo filename_out.
+ *
+ *  Cuando encuentra una coincidencia en la posición p0 del archivo, continua la
+ *  siguiente busqueda en p0+strlen(pat).
+ *  \param[in] filename_out Archivo de salida donde se retornará el resultado del remplazo.
+ *  \param[in] filename_in Archivo de entrada donde se realizará la búsqueda.
+ *  \param[in] pat Patron a buscar.
+ *  \param[in] rep Patron a de remplazo.
+ *  \return Retorna TRUE si todo fue bien o FALSE si no.
+ *  \ingroup PdsFilesFuncGroup
+ */
+int pds_file_pattern_replacement(const char *filename_out,const char *filename_in,const char *pat,const char *rep)
+{
+    char *str_in=NULL;
+    char *str_out=NULL;
+    FILE *fd=NULL;
+
+    if(filename_out==NULL)   return FALSE;
+    if(filename_in==NULL)    return FALSE;
+    if(pat==NULL)           return FALSE;
+    if(rep==NULL)           return FALSE;
+
+
+    str_in=pds_string_new_from_file(filename_in);
+    if(str_in==NULL)    return FALSE;
+
+    str_out=pds_string_new_with_replacement(str_in,pat,rep);
+    if(str_out==NULL)
+    {
+        free(str_in);
+        return FALSE;
+    }
+
+    fd=fopen(filename_out,"w");
+    if(fd==NULL)
+    {
+        free(str_in);
+        free(str_out);
+        return FALSE;
+    }
+    fprintf(fd,"%s",str_out);
+    fclose(fd);
+
+    free(str_in);
+    free(str_out);
+
+    return TRUE;
+}
+
+
+/** \fn int pds_file_content_replacement(   const char *filename_out,const char *filename_in,const char *pat_open,const char *pat_close,char *(*func)(const char *))
+ *  \brief Esta función busca en un archivo de texto filename_in, el contenido
+ *  entre un patrón pat_open y pat_close y lo remplaza  por el valor devuelto de 
+ *  la función func, el resultado de todo esto es escrito en un archivo filename_out.
+ *
+ *  La función func debe ser de la forma :
+\code{.c}
+char *func(const char* content);
+\endcode
+ *
+\code{.c}
+pds_file_content_replacement("output.html","input.html","<b>","</b>",func);
+\endcode
+ *  \param[in] filename_out Archivo de salida donde se retornará el resultado del remplazo.
+ *  \param[in] filename_in Archivo de entrada donde se realizará la búsqueda.
+ *  \param[in] pat_open Patrón inicial a buscar.
+ *  \param[in] pat_close Patrón final a buscar.
+ *  \param[in] func Es una función que se usará para remplazar el contenido 
+ *  encontrado. Por ejemplo si entre pat_open y pat_close es encontrado content
+\code{.c}
+replace=func(content);
+\endcode
+ *  este será remplazado por el contenido de la cadena replace, luego la cadena 
+ *  replace será liberada internamente con free(replace).
+ *  \return Retorna TRUE si todo fue bien o FALSE si no.
+ *  \ingroup PdsFilesFuncGroup
+ */
+int pds_file_content_replacement(   const char *filename_out,const char *filename_in,
+                                    const char *pat_open,const char *pat_close,
+                                    char *(*func)(const char *))
+{
+    FILE *fd=NULL;
+    char *str_in=NULL;
+
+
+    char *content=NULL;
+    char *rep=NULL;
+    char *str_ptr=NULL;
+    char *str_last=NULL;
+    long N,Nopen,Nclose,Ncontent,i,n;
+
+    if(filename_out==NULL)   return FALSE;
+    if(filename_in==NULL)    return FALSE;
+    if(pat_open==NULL)       return FALSE;
+    if(pat_close==NULL)      return FALSE;
+    if((*func)==NULL)        return FALSE;
+
+    str_in= pds_string_new_from_file(filename_in);
+    if(str_in==NULL)    return FALSE;
+
+    str_ptr=str_in;
+    N=strlen(str_in);
+    Nopen=strlen(pat_open);
+    Nclose=strlen(pat_close);
+
+    fd=fopen(filename_out,"w");
+    if(fd==NULL)
+    {
+        free(str_in);
+        return FALSE;
+    }
+
+
+    while(1)
+    {
+        str_last=str_ptr;
+
+        content=pds_string_new_rawdata_after_match(str_ptr,pat_open,pat_close,&str_ptr);
+        
+        if(content!=NULL)
+        {
+            Ncontent=strlen(content);
+            n=( (long)(str_ptr-str_last) )-Ncontent-Nclose;
+            for(i=0;i<n;i++)    fputc(str_last[i],fd);
+
+            rep=(*func)(content);
+
+            fputs(rep,fd);
+            fputs(pat_close,fd);
+            
+            free(content);
+            free(rep);
+        }
+        else
+        {
+            fputs(str_last,fd);
+            break;
+        }
+
+    }
+    fclose(fd); 
+  
+    return TRUE;
+}
